package com.hz.pm.api.common.statemachine.builder.impl;

import com.hz.pm.api.common.statemachine.builder.TenderStateMachineBuilder;
import com.hz.pm.api.common.statemachine.event.TenderStatusChangeEvent;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectlib.model.enumeration.TenderStatusEnum;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.statemachine.StateMachine;
import org.springframework.statemachine.StateMachineContext;
import org.springframework.statemachine.StateMachineException;
import org.springframework.statemachine.StateMachinePersist;
import org.springframework.statemachine.config.EnableStateMachine;
import org.springframework.statemachine.config.StateMachineBuilder;
import org.springframework.statemachine.persist.DefaultStateMachinePersister;
import org.springframework.statemachine.persist.StateMachinePersister;
import org.springframework.statemachine.support.DefaultStateMachineContext;
import org.springframework.stereotype.Component;

import java.util.EnumSet;

/**
 * <p>
 * 标段状态机
 * </p>
 *
 * @author WendyYang
 * @since 16:38 2024/2/24
 */
@Slf4j
@Component
@EnableStateMachine(name = TenderStateMachineBuilderImpl.MACHINE_ID)
@RequiredArgsConstructor
public class TenderStateMachineBuilderImpl implements TenderStateMachineBuilder {

    public static final String MACHINE_ID = "tenderStateMachine";

    private final BeanFactory beanFactory;

    @Override
    public StateMachine<TenderStatusEnum, TenderStatusChangeEvent> build() throws StateMachineException {
        StateMachine<TenderStatusEnum, TenderStatusChangeEvent> stateMachine = build(beanFactory);
        log.info("状态机ID：" + stateMachine.getId());
        stateMachine.start();
        return stateMachine;
    }

    @Override
    public StateMachine<TenderStatusEnum, TenderStatusChangeEvent> build(BeanFactory beanFactory) throws StateMachineException {
        try {
            return buildStateMachine(beanFactory);
        } catch (Exception e) {
            throw new StateMachineException("状态机构件失败", e);
        }
    }

    private StateMachine<TenderStatusEnum, TenderStatusChangeEvent> buildStateMachine(BeanFactory factory) throws Exception {
        StateMachineBuilder.Builder<TenderStatusEnum, TenderStatusChangeEvent> builder = StateMachineBuilder.builder();
        builder.configureConfiguration()
                .withConfiguration()
                .machineId(MACHINE_ID)
                .beanFactory(factory);

        builder.configureStates()
                .withStates()
                .initial(TenderStatusEnum.TO_BE_SUBMIT_PURCHASE_INFO)
                .states(EnumSet.allOf(TenderStatusEnum.class));

        builder.configureTransitions()
                // 填写采购信息
                .withExternal()
                .source(TenderStatusEnum.TO_BE_SUBMIT_PURCHASE_INFO)
                .target(TenderStatusEnum.TO_BE_SUBMIT_CONSTRUCTION_INFO)
                .event(TenderStatusChangeEvent.SUBMIT_PURCHASE_INFO)
                .and()
                // 填写合同信息
                .withExternal()
                .source(TenderStatusEnum.TO_BE_SUBMIT_CONSTRUCTION_INFO)
                .target(TenderStatusEnum.TO_BE_SUBMIT_OPERATION_PLAN)
                .event(TenderStatusChangeEvent.SUBMIT_CONSTRUCTION_INFO)
                .and()
                // 填写实施计划
                .withExternal()
                .source(TenderStatusEnum.TO_BE_SUBMIT_OPERATION_PLAN)
                .target(TenderStatusEnum.TO_BE_SUBMIT_FIRST_INSPECTED_INFO)
                .event(TenderStatusChangeEvent.SUBMIT_OPERATION_PLAN)
                .and()
                // 上传初验材料
                .withExternal()
                .source(TenderStatusEnum.TO_BE_SUBMIT_FIRST_INSPECTED_INFO)
                .target(TenderStatusEnum.TO_BE_SUBMIT_XCFHX_APPLY)
                .event(TenderStatusChangeEvent.SUBMIT_FIRST_INSPECTED_FILES)
                .and()
                // 信创符合性申请
                .withExternal()
                .source(TenderStatusEnum.TO_BE_SUBMIT_XCFHX_APPLY)
                .target(TenderStatusEnum.ON_XCFHX_APPLY)
                .event(TenderStatusChangeEvent.SUBMIT_XCFHX_APPLY)
                .and()// 信创符合性申请
                .withExternal()
                .source(TenderStatusEnum.XCFHX_APPLY_FAILED)
                .target(TenderStatusEnum.ON_XCFHX_APPLY)
                .event(TenderStatusChangeEvent.RESUBMIT_XCFHX_APPLY)
                .and()
                // 信创符合性申请通过
                .withExternal()
                .source(TenderStatusEnum.ON_XCFHX_APPLY)
                .target(TenderStatusEnum.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY)
                .event(TenderStatusChangeEvent.XCFHX_APPLY_PASSED)
                .and()
                // 信创符合性申请失败
                .withExternal()
                .source(TenderStatusEnum.ON_XCFHX_APPLY)
                .target(TenderStatusEnum.XCFHX_APPLY_FAILED)
                .event(TenderStatusChangeEvent.XCFHX_APPLY_FAILED)
                .and()
                // 终验申报
                .withExternal()
                .source(TenderStatusEnum.TO_BE_SUBMIT_FINALLY_INSPECTED_APPLY)
                .target(TenderStatusEnum.ON_FINALLY_INSPECTED_APPLY)
                .event(TenderStatusChangeEvent.SUBMIT_FINALLY_INSPECTED)
                .and()
                // 终验申报重新提交
                .withExternal()
                .source(TenderStatusEnum.FINALLY_INSPECTED_FAILED)
                .target(TenderStatusEnum.ON_FINALLY_INSPECTED_APPLY)
                .event(TenderStatusChangeEvent.RESUBMIT_FINALLY_INSPECTED)
                .and()
                // 终验申报通过
                .withExternal()
                .source(TenderStatusEnum.ON_FINALLY_INSPECTED_APPLY)
                .target(TenderStatusEnum.FINALLY_INSPECTED_PASSED)
                .event(TenderStatusChangeEvent.FINALLY_INSPECTED_PASSED)
                .and()
                // 终验申报拒绝
                .withExternal()
                .source(TenderStatusEnum.ON_FINALLY_INSPECTED_APPLY)
                .target(TenderStatusEnum.FINALLY_INSPECTED_FAILED)
                .event(TenderStatusChangeEvent.FINALLY_INSPECTED_FAILED)
                .and();
        return builder.build();
    }

    @Override
    @Bean(name = "tenderStatePersister")
    public StateMachinePersister<TenderStatusEnum, TenderStatusChangeEvent, Purchase> stateMachinePersister() {
        return new DefaultStateMachinePersister<>(new StateMachinePersist<TenderStatusEnum, TenderStatusChangeEvent, Purchase>() {
            @Override
            public void write(StateMachineContext<TenderStatusEnum, TenderStatusChangeEvent> context, Purchase contextObj) {
                log.info("当前项目为：{}", contextObj);
            }

            @Override
            public StateMachineContext<TenderStatusEnum, TenderStatusChangeEvent> read(Purchase contextObj) {
                return new DefaultStateMachineContext<>(TenderStatusEnum.getByStatus(contextObj.getStatus()),
                        null, null, null, null, MACHINE_ID);
            }
        });
    }

}
