package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Maps;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.model.constant.RegionConst;
import com.hz.pm.api.common.model.constant.TypeReferenceConst;
import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectdeclared.model.dto.ProjectConditionDTO;
import com.hz.pm.api.projectdeclared.model.po.DeclaredProjectStatisticsPO;
import com.hz.pm.api.projectdeclared.service.IDeclaredStatisticsService;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.dto.ProjectPaymentPlanDTO;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.ningdatech.basic.function.VUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.bean.entity.WflowOrgModels;
import com.wflow.enums.OrgTypeEnum;
import com.wflow.enums.ProcessDefTypeEnum;
import com.wflow.exception.BusinessException;
import com.wflow.service.OrgProcdefService;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.dto.ProcessInstanceUserDto;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessModelService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Classname DefaultDeclaredProjectManage
 * @Description
 * @Date 2023/2/14 17:12
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class DefaultDeclaredProjectManage {

    private final IProjectService projectService;

    private final UserInfoHelper userInfoHelper;

    private final ProcessModelService processModelService;
    private final OrgProcdefService orgProcdefService;

    private final IDeclaredStatisticsService statisticsService;

    /**
     * 公共的发起流程方法
     */
    public void startProcess(ProjectDTO projectDTO, UserFullInfoDTO user, Integer processType) {
        WflowModels model = processModelService.getOne(Wrappers.lambdaQuery(WflowModels.class)
                .eq(WflowModels::getProcessType, processType)
                .last(BizConst.LIMIT_1));

        if (Objects.isNull(model)) {
            throw new BusinessException("找不到对应流程配置");
        }

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(buildUser(user));
        params.setProcessUsers(Collections.emptyMap());
        //放入条件判断的项目字段
        ProjectConditionDTO conditionDto = new ProjectConditionDTO();
        BeanUtils.copyProperties(projectDTO, conditionDto);
        Map<String, Object> formData = Maps.newHashMap();
        formData.putAll(JSON.parseObject(JSON.toJSONString(conditionDto), TypeReferenceConst.STR_OBJ_MAP));
        params.setFormData(formData);

    }

    //项目名称去重
    public void checkDuplication(ProjectDTO project) {
        VUtils.isTrue(projectService.count(Wrappers.lambdaQuery(Project.class)
                        .eq(Project::getProjectName, project.getProjectName())
                        .eq(Project::getNewest, Boolean.TRUE)
                        .ne(Objects.nonNull(project.getProjectCode()), Project::getProjectCode, project.getProjectCode())) > 0)
                .throwMessage(String.format("修改失败 此项目名 【%s】 已存在!", project.getProjectName()));
    }

    public void checkAmount(ProjectDTO projectInfo) {
        if (Objects.nonNull(projectInfo.getDeclareAmount())) {
            BigDecimal declareAmount = projectInfo.getDeclareAmount();
            BigDecimal govSuperFinanceAmount = projectInfo.getDeclareGovSuperiorFinanceAmount();
            BigDecimal govOwnFinanceAmount = projectInfo.getDeclareGovOwnFinanceAmount();
            BigDecimal bankLendingAmount = projectInfo.getDeclareBankLendingAmount();
            BigDecimal haveAmount = projectInfo.getDeclareHaveAmount();
            BigDecimal otherAmount = projectInfo.getDeclareOtherAmount();
            BigDecimal totalAmount = govSuperFinanceAmount
                    .add(govOwnFinanceAmount).add(bankLendingAmount).add(haveAmount).add(otherAmount);
            VUtils.isTrue(declareAmount.compareTo(totalAmount) != 0)
                    .throwMessage(String.format("申报失败! 申报总金额【%s】 不等于其它申报金额 【%s】", declareAmount, totalAmount));

            //判断 年度计划金额 如果开着的话
            if (CollUtil.isNotEmpty(projectInfo.getPaymentPlanList())) {
                for (ProjectPaymentPlanDTO paymentPlan : projectInfo.getPaymentPlanList()) {
                    BigDecimal annualPlanAmount = paymentPlan.getAnnualPlanAmount();

                    VUtils.isTrue(annualPlanAmount.compareTo(BigDecimal.ZERO) <= 0)
                            .throwMessage(String.format("申报失败! 年度支付总金额【%s】 不能小于等于0", annualPlanAmount));
                    VUtils.isTrue(annualPlanAmount.compareTo(declareAmount) > 0)
                            .throwMessage(String.format("申报失败! 年度支付总金额【%s】 不能大于申报金额 【%s】", annualPlanAmount, declareAmount));

                    BigDecimal annualGovSuperAmount = paymentPlan.getAnnualPlanGovSuperiorFinanceAmount();
                    BigDecimal annualOwnSuperAmount = paymentPlan.getAnnualPlanGovOwnFinanceAmount();
                    BigDecimal annualHaveAmount = paymentPlan.getAnnualPlanHaveAmount();
                    BigDecimal totalAnnual = annualGovSuperAmount.add(annualOwnSuperAmount).add(annualHaveAmount);
                    VUtils.isTrue(annualPlanAmount.compareTo(totalAnnual) != 0)
                            .throwMessage(String.format("申报失败! 年度支付总金额【%s】 不等于其它年度支付金额 【%s】", annualPlanAmount, totalAnnual));

                }
            }
        }
    }

    public ProcessInstanceUserDto buildUser(UserFullInfoDTO user) {
        return ProcessInstanceUserDto.builder()
                .userId(user.getUserIdStr())
                .userName(user.getRealName())
                .orgCode(user.getMhUnitIdStr())
                .orgName(user.getMhUnitName())
                .build();
    }

    public ProcessInstanceUserDto buildUser(Long userId) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);
        Assert.notNull(user, "【%s】获取不到用户!", userId);
        return buildUser(user);
    }

    public ProcessInstanceUserDto buildUser(UserInfoDetails user) {
        return ProcessInstanceUserDto.builder()
                .userId(user.getUserIdStr())
                .userName(user.getRealName())
                .orgCode(user.getMhUnitIdStr())
                .orgName(user.getMhUnitName())
                .build();
    }

    public Map<String, OrgInfoDTO> buildOrgModelMap(Long userId, Project project) {
        Map<String, OrgInfoDTO> orgMap = new HashMap<>();

        // 查出所有的单位流程配置
        List<WflowOrgModels> orgModelsList = orgProcdefService.list();

        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        if (Objects.isNull(userFullInfo)) {
            log.warn("取不到员工 返回空map");
            return Collections.emptyMap();
        }

        String startOrgCode = userFullInfo.getMhUnitIdStr();
        String startOrgName = userFullInfo.getMhUnitName();
        String startOrgParentCode;
        String startOrgParentName;
        // 如果申报时 上级主管单位,由该单位自己审核
        if (CommonEnum.NO.getCode().equals(project.getIsSuperOrg())) {
            startOrgParentCode = startOrgCode;
            startOrgParentName = startOrgName;
        } else {
            startOrgParentCode = project.getSuperOrgCode();
            startOrgParentName = project.getSuperOrg();
        }
        //查询 当前发起人及主管单位所在区域的 单位流程配置
        OrgInfoDTO startOrgInfoDto = new OrgInfoDTO();
        startOrgInfoDto.setOrganizationCode(startOrgCode);
        startOrgInfoDto.setOrganizationName(startOrgName);

        startOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                .filter(v -> v.getOrgCode().equals(startOrgCode)
                        && Boolean.FALSE.equals(v.getIsDelete())
                        && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.JOINT_REVIEW.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

        OrgInfoDTO parentOrgInfoDto = new OrgInfoDTO();
        parentOrgInfoDto.setOrganizationCode(startOrgParentCode);
        parentOrgInfoDto.setOrganizationName(startOrgParentName);
        parentOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                .filter(v -> v.getOrgCode().equals(startOrgParentCode)
                        && Boolean.FALSE.equals(v.getIsDelete())
                        && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.JOINT_REVIEW.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

        orgMap.put(OrgTypeEnum.TARGET_OWNER.name(), startOrgInfoDto);
        orgMap.put(OrgTypeEnum.TARGET_MANAGEMENT.name(), parentOrgInfoDto);

        // 如果是指定单位，直接根据流程定义ID放入map
        OrgInfoDTO orgInfoDTO = new OrgInfoDTO();
        orgInfoDTO.setOrgModelMap(orgModelsList.stream()
                .collect(Collectors.toMap(WflowOrgModels::getProcessDefId, v -> v)));
        orgMap.put(OrgTypeEnum.TARGET_LABEL.name(), orgInfoDTO);

        //如果有上级条线主管单位 并且 不是市本级
        if (!RegionConst.RC_HZ.equals(project.getAreaCode()) &&
                CommonEnum.YES.getCode().equals(project.getIsHigherSuperOrg())) {
            OrgInfoDTO parentLineOrgInfoDto = new OrgInfoDTO();
            parentLineOrgInfoDto.setOrganizationCode(project.getHigherSuperOrgCode());
            parentLineOrgInfoDto.setOrganizationName(project.getHigherSuperOrg());
            parentLineOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                    .filter(v -> v.getOrgCode().equals(project.getHigherSuperOrgCode())
                            && Boolean.FALSE.equals(v.getIsDelete())
                            && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                            ProcessDefTypeEnum.JOINT_REVIEW.name().equals(v.getType()) ||
                            ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                    .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

            orgMap.put(OrgTypeEnum.TARGET_LINE_MANAGEMENT.name(), parentLineOrgInfoDto);
        }
        return orgMap;
    }

    public Map<String, OrgInfoDTO> buildOrgModelMapUserOrgCode(Project project) {
        Map<String, OrgInfoDTO> orgMap = new HashMap<>();

        // 查出所有的单位流程配置
        List<WflowOrgModels> orgModelsList = orgProcdefService.list();

        String startOrgCode = project.getBuildOrgCode();
        String startOrgName = project.getBuildOrgName();
        String startOrgParentCode;
        String startOrgParentName;
        // 如果申报时 上级主管单位,由该单位自己审核
        if (CommonEnum.NO.getCode().equals(project.getIsSuperOrg())) {
            startOrgParentCode = startOrgCode;
            startOrgParentName = startOrgName;
        } else {
            startOrgParentCode = project.getSuperOrgCode();
            startOrgParentName = project.getSuperOrg();
        }
        //查询 当前发起人及主管单位所在区域的 单位流程配置
        OrgInfoDTO startOrgInfoDto = new OrgInfoDTO();
        startOrgInfoDto.setOrganizationCode(startOrgCode);
        startOrgInfoDto.setOrganizationName(startOrgName);

        startOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                .filter(v -> v.getOrgCode().equals(startOrgCode)
                        && Boolean.FALSE.equals(v.getIsDelete())
                        && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.JOINT_REVIEW.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

        OrgInfoDTO parentOrgInfoDto = new OrgInfoDTO();
        parentOrgInfoDto.setOrganizationCode(startOrgParentCode);
        parentOrgInfoDto.setOrganizationName(startOrgParentName);
        parentOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                .filter(v -> v.getOrgCode().equals(startOrgParentCode)
                        && Boolean.FALSE.equals(v.getIsDelete())
                        && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.JOINT_REVIEW.name().equals(v.getType()) ||
                        ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

        orgMap.put(OrgTypeEnum.TARGET_OWNER.name(), startOrgInfoDto);
        orgMap.put(OrgTypeEnum.TARGET_MANAGEMENT.name(), parentOrgInfoDto);

        // 如果是指定单位，直接根据流程定义ID放入map
        OrgInfoDTO orgInfoDTO = new OrgInfoDTO();
        orgInfoDTO.setOrgModelMap(orgModelsList.stream()
                .collect(Collectors.toMap(WflowOrgModels::getProcessDefId, v -> v)));
        orgMap.put(OrgTypeEnum.TARGET_LABEL.name(), orgInfoDTO);

        //如果有上级条线主管单位 并且 不是市本级
        if (!RegionConst.RC_HZ.equals(project.getAreaCode()) &&
                CommonEnum.YES.getCode().equals(project.getIsHigherSuperOrg())) {
            OrgInfoDTO parentLineOrgInfoDto = new OrgInfoDTO();
            parentLineOrgInfoDto.setOrganizationCode(project.getHigherSuperOrgCode());
            parentLineOrgInfoDto.setOrganizationName(project.getHigherSuperOrg());
            parentLineOrgInfoDto.setOrgModelMap(orgModelsList.stream()
                    .filter(v -> v.getOrgCode().equals(project.getHigherSuperOrgCode())
                            && Boolean.FALSE.equals(v.getIsDelete())
                            && (ProcessDefTypeEnum.SEAL.name().equals(v.getType()) ||
                            ProcessDefTypeEnum.JOINT_REVIEW.name().equals(v.getType()) ||
                            ProcessDefTypeEnum.DEFAULT.name().equals(v.getType())))
                    .collect(Collectors.toMap(WflowOrgModels::getType, v -> v)));

            orgMap.put(OrgTypeEnum.TARGET_LINE_MANAGEMENT.name(), parentLineOrgInfoDto);
        }
        return orgMap;
    }

    //根据提交者的单位
    public DeclaredProjectStatisticsPO declaredProjectOrgStatistics(Integer year, UserFullInfoDTO user) {
        //查此人建设单位的项目
        return statisticsService.getOrgStatistics(user.getMhUnitIdStr(), year);
    }

    //根据提交者的区域 他是区管或者超管
    public DeclaredProjectStatisticsPO declaredProjectRegionStatistics(Integer year, UserFullInfoDTO user) {
        //查此人建设单位的项目
        return statisticsService.getRegionStatistics(null, year);
    }

    /**
     * 把条件值放入工作流 用于判断条件 根据dto
     *
     * @param params
     * @param dto
     */
    public void buildCondition(ProcessStartParamsVo params, DefaultDeclaredDTO dto) {
        //放入条件判断的项目字段
        ProjectConditionDTO conditionDto = new ProjectConditionDTO();
        BeanUtils.copyProperties(dto.getProjectInfo(), conditionDto);
        dto.getFormData().putAll(
                JSON.parseObject(JSON.toJSONString(conditionDto), new TypeReference<Map<String, Object>>() {
                })
        );
        params.setFormData(dto.getFormData());
    }

    /**
     * 把条件值放入工作流 用于判断条件 根据d项目实体
     *
     * @param params
     * @param projectInfo
     */
    public void buildCondition(ProcessStartParamsVo params, Project projectInfo) {
        //放入条件判断的项目字段
        ProjectConditionDTO conditionDto = new ProjectConditionDTO();
        BeanUtils.copyProperties(projectInfo, conditionDto);
        params.setFormData(JSON.parseObject(JSON.toJSONString(conditionDto), new TypeReference<Map<String, Object>>() {
        }));
    }

    public void buildCondition(ProcessStartParamsVo params, Project projectInfo, DefaultDeclaredDTO dto) {
        //放入条件判断的项目字段 优先取dto的 再取项目的
        ProjectDTO projInfo = dto.getProjectInfo();
        ProjectConditionDTO condition = new ProjectConditionDTO();
        condition.setProjectId(projectInfo.getId());
        condition.setDeclareAmount(Objects.nonNull(projInfo.getDeclareAmount())
                ? projInfo.getDeclareAmount() : projectInfo.getDeclareAmount());
        condition.setIsDigitalReform(Objects.nonNull(projInfo.getIsDigitalReform())
                ? projInfo.getIsDigitalReform() : projectInfo.getIsDigitalReform());
        condition.setIsTemporaryAugment(Objects.nonNull(projInfo.getIsTemporaryAugment())
                ? projInfo.getIsTemporaryAugment() : projectInfo.getIsTemporaryAugment());
        condition.setBizDomain(Objects.nonNull(projInfo.getBizDomain())
                ? projInfo.getBizDomain() : projectInfo.getBizDomain());
        params.setFormData(JSON.parseObject(JSON.toJSONString(condition), TypeReferenceConst.STR_OBJ_MAP));
    }
}
