package com.hz.pm.api.todocenter.utils;


import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.BaseFont;
import com.ningdatech.basic.exception.BizException;
import lombok.extern.slf4j.Slf4j;
import org.xhtmlrenderer.pdf.ITextFontResolver;
import org.xhtmlrenderer.pdf.ITextRenderer;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

/**
 * pdf生成工具类
 *
 * @author CMM
 * @since 2023/03/13 17:01
 */
@Slf4j
public class PdfUtil {

    private PdfUtil() {
    }

    /**
     * 生成PDF文件
     *
     * @return int
     * @author CMM
     * @since 2023/03/13 17:07
     */
    public static byte[] generatePdf(InputStream templateHtmlInputStream, Map<String, Object> paramsMap) {
        try (BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(templateHtmlInputStream, StandardCharsets.UTF_8))) {
            // 字体格式
            String fontPath = "font" + File.separator + "simsun.ttc";
            StringBuilder stringBuilder = new StringBuilder();
            bufferedReader.lines().forEach(stringBuilder::append);
            String htmlInfo = stringBuilder.toString();

            // 替换参数、多个参数多次替换
            for (Map.Entry<String, Object> entry : paramsMap.entrySet()) {
                String value = Optional.ofNullable(entry.getValue())
                        .map(Object::toString)
                        .orElse("");
                htmlInfo = htmlInfo.replace("#" + entry.getKey() + "#", value);
            }

            // 生成临时文件
            String htmlFileName = UUID.randomUUID().toString().replace("-", "") + ".html";
            String htmlFilePath = "fileout" + File.separator + htmlFileName;
            File file = new File(htmlFilePath);
            try (FileOutputStream fop = new FileOutputStream(file)) {
                byte[] contentInBytes = htmlInfo.getBytes();
                fop.write(contentInBytes);
            } catch (IOException e) {
                throw BizException.wrap("生成html文件失败!" + e.getMessage());
            }

            // 生成pdf
            String pdfFileName = UUID.randomUUID().toString().replace("-", "") + ".pdf";
            String pdfFilePath = "fileout" + File.separator + pdfFileName;
            String url = new File(htmlFilePath).toURI().toURL().toString();
            try (OutputStream os = Files.newOutputStream(Paths.get(pdfFilePath))) {
                ITextRenderer renderer = new ITextRenderer();
                renderer.setDocument(url);
                ITextFontResolver fontResolver = renderer.getFontResolver();
                fontResolver.addFont(fontPath, BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                renderer.layout();
                renderer.createPDF(os);
            } catch (IOException | DocumentException e) {
                throw BizException.wrap("生成pdf文件失败!" + e.getMessage());
            }

            // 返回生成的pdf文件字节数组
            try (FileInputStream inputStream = new FileInputStream(pdfFilePath)) {
                return readBytes(inputStream);
            } catch (IOException e) {
                throw BizException.wrap("读取pdf文件失败!" + e.getMessage());
            }
        } catch (IOException e) {
            throw BizException.wrap("读取模板html文件失败!" + e.getMessage());
        }
    }


    /**
     * 读取输入流到字节数组
     *
     * @param in
     * @return
     * @throws IOException
     */
    private static byte[] readBytes(InputStream in) throws IOException {
        //读取字节的缓冲
        byte[] buffer = new byte[1024];
        //最终的数据
        byte[] result = new byte[0];
        int size;
        while ((size = in.read(buffer)) != -1) {
            int oldLen = result.length;
            byte[] tmp = new byte[oldLen + size];
            if (oldLen > 0) {
                //copy 旧字节
                System.arraycopy(result, 0, tmp, 0, oldLen);
            }
            //copy 新字节
            System.arraycopy(buffer, 0, tmp, oldLen, size);
            result = tmp;
        }
        return result;
    }

}
